package GameEngine.Core;

/**
 * responsible for create general messages that are used in all modes.
 * the messages that are specific to mode will be created in GameMode class or child class 
 * @author tranlongtien
 *
 */
public class Message {
	protected char msgType;
	protected String msgContent;
	protected String msgRaw;
	
	public static final int MAX_NUMBER_OF_FIELDS_IN_MSG = 4000; // maximum number of unique fields in a message. do not change
	
    /**
     * These characters are prepended to messages that are sent to Game Engine, to show what type of messages they are
     */
	
	public static final char MSG_UNKNOWN_MESSAGE = '_';
	public static final char MSG_NUMBER_OF_QUES = '0';
	public static final char MSG_GAME_MODE = '1';
	public static final char MSG_START = '2'; // Start the game
	public static final char MSG_EXIT = '3'; // Exit game
	public static final char MSG_LECTURE_ID = '4'; // message to set lecture ID.
	public static final char MSG_END_OF_GAME = '5'; // message sent to client when reaching end of game.
	public static final char MSG_END_OF_ROUND = '6';
    
	public static final char MSG_QUESTION ='7';
	public static final char MSG_ANSWER ='8';
	public static final char MSG_LOGIN ='9';
	public static final char MSG_MOBILE_LOGIN ='A';
	public static final char MSG_NEXT_QUESTION ='B';
   
    /**
     * Handshake string. Each end of the connection sends this  string to the 
     * other just after the connection is opened.  This is done to confirm that 
     * the program on the other side of the connection is a proper program.
     */

    public static final String HANDSHAKE_MASTER = "MasterServer";
    public static final String HANDSHAKE_MOBILE = "MobileServer";
    
    public Message(String rawMessage)
    {
    	msgRaw = rawMessage;
    }

    
    public char getTypeOfMsg()
    {
    	char type;
    	if (msgRaw.length() >0)
    	{
    		type = msgRaw.charAt(0);
    		return type;
    	}
    	return MSG_UNKNOWN_MESSAGE;
    }
    
    public String getMsgContent()
    {
    	if (msgRaw.length() >1)
    		return msgRaw.substring(1);
    	return null;
    }
    
    public String getRawMessage()
    {
    	return msgRaw;
    }
    
	/**
     * Creates the root of the question to be sendt to eighter the studenclients or the masterclient. The
     * root is similar, thus this only needs one method. The last part of the question is different though.
     *
     * Example of output:
     *
     * 7Who am I?|3|me|you|them|30|
     *
     * this is translated to
     * question: who am I?
     * number of alternatives: 3
     * alternative 1: me
     * alternative 2: you
     * alternative 3: them
     * timelimit: 30 s
     *
     * @return  a question root
     */
    private String makeQuestionRoot(Question q) {
    	String returnQuestion;
    	returnQuestion = q.getformattedQuestion();
        if (returnQuestion != null) return returnQuestion;
        
        returnQuestion = Message.MSG_QUESTION + q.getQuestion() + "|" + Integer.toString(q.getNumberOfAlt());
        for (String alternative : q.getAlternatives()) returnQuestion += "|" + alternative;
        return returnQuestion += "|" + q.getTimeLimit() +"|";
    }
    
    /**
     * Creates the question to be sendt to the StudentClient. A
     */
    public String makeClientQuestion(Question q){

        String qstString = makeQuestionRoot(q) + q.getCorrectAlternative();
        return qstString;
    }

    /**
     * Creates the question to be sendt to the MasterClient. 
     */
    public String makeMasterQuestion(Question q) {

        String qstString = makeQuestionRoot(q);
        StringBuilder masterQst = new StringBuilder(qstString);
        masterQst.append(q.getCorrectAlternative()); // just the index of the correct answer
        return masterQst.toString();
    }
    
    
    
    public static String createExitMessage()
    {
    	String ret;
    	ret = Message.MSG_EXIT + "";
    	return ret; 
    }
    
    public static String createEndGameMessage()
    {
    	String ret;
    	ret = Message.MSG_END_OF_GAME + "";
    	return ret;
    }

    public static String createStartGameMessage()
    {
    	String ret;
    	ret = Message.MSG_START + "";
    	return ret;
    }
    
    public static String createLoginOk(){
    	return Message.MSG_LOGIN + "ok";
    }
    
    public static String createEndGame4Mobile(MobileCommunicator c){
    	String ret;
    	ret = Message.MSG_END_OF_GAME +"";
    	ret = ret + c.getTotalScore() + "|" + c.getMaxScore();
    	return ret;
    }
    
    
	public  void printMsg(){
		System.out.println("This message is printed from Message class: ");
		System.out.println ("msgraw: " + msgRaw + ", msgcontent: " + msgContent + ", type: " + msgType);		
	}
    
    /**
     * check if the message received is an answer from mobile
     * 
     */
    public boolean isAnswer()
    {
    	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_ANSWER);
    }
    
    public int getAnswerIndex(){
    	int ret =-1;
    	ret = Integer.parseInt(getMsgContent());
    	return ret;
    }
    
    public static String createMobileLoginMsg(String user){
    	return (MSG_MOBILE_LOGIN + user);
    }
    
}// End class
